@@ -10,18 +10,18 @@ use constant SPECIFICATION_URL => 'http://swagger.io/v2/schema.json';
 
 my %COLLECTION_RE = (pipes => qr{\|}, csv => qr{,}, ssv => qr{\s}, tsv => qr{\t});
 
-has _json_validator => sub { state $v = JSON::Validator->new; };
+has _json_validator => sub { state $v = shift->new; };
 
 sub load_and_validate_schema {
   my ($self, $spec, $args) = @_;
-  my $openapi = JSON::Validator->new->schema($args->{schema} || SPECIFICATION_URL);
+  my $openapi = $self->new->schema($args->{schema} || SPECIFICATION_URL);
   my ($api_spec, @errors);
 
   # 1. first check if $ref is in the right place,
   # 2. then check if the spec is correct
   for my $r (sub { }, undef) {
     next if $r and $args->{allow_invalid_ref};
-    my $jv = JSON::Validator->new;
+    my $jv = $self->new;
     $jv->resolver($r) if $r;
     $api_spec = $jv->schema($spec)->schema;
     @errors   = $openapi->coerce($jv->coerce)->validate($api_spec->data);
@@ -51,7 +51,7 @@ sub validate_input {
 
 sub validate_request {
   my ($self, $c, $schema, $input) = @_;
-  my (%cache, @errors);
+  my @errors;
 
   for my $p (@{$schema->{parameters} || []}) {
     my ($in, $name, $type) = @$p{qw(in name type)};
@@ -62,11 +62,11 @@ sub validate_request {
       $exists = length $value if defined $value;
     }
     elsif ($in eq 'formData' and $type eq 'file') {
-      ($value) = $self->_get_request_uploads($c, $name);
+      $value = $self->_get_request_uploads($c, $name)->[-1];
       $exists = $value ? 1 : 0;
     }
     else {
-      $value  = $cache{$in} ||= $self->_get_request_data($c, $in);
+      $value  = $self->_get_request_data($c, $in);
       $exists = exists $value->{$name};
       $value  = $value->{$name};
     }
@@ -76,6 +76,9 @@ sub validate_request {
     }
 
     if ($type and defined($value //= $p->{default})) {
+      if ($type ne 'array' and ref $value eq 'ARRAY') {
+        $value = $value->[-1];
+      }
       if (($type eq 'integer' or $type eq 'number') and Scalar::Util::looks_like_number($value)) {
         $value += 0;
       }
@@ -286,10 +289,9 @@ sub _coerce_by_collection_format {
   return $single ? $data->[0] : $data;
 }
 
-sub _invalid_in {
-  my ($self, $in) = @_;
+sub _confess_invalid_in {
   Carp::confess(
-    "Unsupported \$in: $in. Please report at https://github.com/jhthorsen/json-validator");
+    "Unsupported \$in: $_[0]. Please report at https://github.com/jhthorsen/json-validator");
 }
 
 sub _is_byte_string { $_[0] =~ /^[A-Za-z0-9\+\/\=]+$/ }
